from ..Utility.Common import *
from typing import List

class VersionGroup(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/version-group/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    
    @property
    def order(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"order",int)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"generation",NamedAPIResource)
    
    @property
    def move_learn_methods(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"move_learn_methods",NamedAPIResource)
    
    @property
    def pokedexes(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokedexes",NamedAPIResource)

    @property
    def regions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"regions",NamedAPIResource)
    
    @property
    def versions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"versions",NamedAPIResource)
    


from ..Utility.Common import *
from typing import List

class TypePokemon:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def slot(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"slot",int)
    
    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon",NamedAPIResource)
      
    
    
class TypeRelations:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def no_damage_to(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"no_damage_to",NamedAPIResource)
    
    @property
    def half_damage_to(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"half_damage_to",NamedAPIResource)
    
    @property
    def double_damage_to(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"double_damage_to",NamedAPIResource)
    
    @property
    def no_damage_from(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"no_damage_from",NamedAPIResource)
    
    @property
    def half_damage_from(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"half_damage_from",NamedAPIResource)
    
    @property
    def double_damage_from(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"double_damage_from",NamedAPIResource)
    
class TypeRelationsPast:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"generation",NamedAPIResource)
    
    @property
    def damage_relations(self) -> Union[TypeRelations,None]:
        return Functions.convert_to_type(self.__json_data,"damage_relations",TypeRelations)

class TypeSpritesNameIcon:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def name_icon(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"name_icon",str)

class TypeSpritesGenerationiii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def colosseum(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"colosseum",TypeSpritesNameIcon)
    
    @property
    def emerald(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"emerald",TypeSpritesNameIcon)
    
    @property
    def firered_leafgreen(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"firered-leafgreen",TypeSpritesNameIcon)
    
    @property
    def ruby_saphire(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"ruby-saphire",TypeSpritesNameIcon)
    
    @property
    def xd(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"xd",TypeSpritesNameIcon)

class TypeSpritesGenerationiv:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def diamond_pearl(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"diamond-pearl",TypeSpritesNameIcon)
    
    @property
    def heartgold_soulsilver(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"heartgold-soulsilver",TypeSpritesNameIcon)
    
    @property
    def platinum(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"platinum",TypeSpritesNameIcon)

class TypeSpritesGenerationv:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def black_2_white_2(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"black-2-white-2",TypeSpritesNameIcon)

    @property
    def black_white(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"black-white",TypeSpritesNameIcon)
    
class TypeSpritesGenerationvi:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def omega_ruby_alpha_sapphire(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"omega-ruby-alpha-sapphire",TypeSpritesNameIcon)

    @property
    def x_y(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"x-y",TypeSpritesNameIcon)

class TypeSpritesGenerationvii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def lets_go_pikachu_lets_go_eevee(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"lets-go-pikachu-lets-go-eevee",TypeSpritesNameIcon)

    @property
    def sun_moon(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"sun-moon",TypeSpritesNameIcon)
    
    @property
    def ultra_sun_ultra_moon(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"ultra-sun-ultra-moon",TypeSpritesNameIcon)
    
class TypeSpritesGenerationviii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def brilliant_diamond_and_shining_pearl(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"brilliant-diamond-and-shining-pearl",TypeSpritesNameIcon)

    @property
    def legends_arceus(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"legends-arceus",TypeSpritesNameIcon)
    
    @property
    def sword_shield(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"sword-shield",TypeSpritesNameIcon)
    
class TypeSpritesGenerationix:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def scarlet_violet(self) -> Union[TypeSpritesNameIcon,None]:
        return Functions.convert_to_type(self.__json_data,"scarlet-violet",TypeSpritesNameIcon)

class TypeSprites:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def generation_iii(self) -> Union[TypeSpritesGenerationiii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-iii",TypeSpritesGenerationiii)
    
    @property
    def generation_iv(self) -> Union[TypeSpritesGenerationiv,None]:
        return Functions.convert_to_type(self.__json_data,"generation-iv",TypeSpritesGenerationiv)
    
    @property
    def generation_v(self) -> Union[TypeSpritesGenerationv,None]:
        return Functions.convert_to_type(self.__json_data,"generation-v",TypeSpritesGenerationv)
    
    @property
    def generation_vi(self) -> Union[TypeSpritesGenerationvi,None]:
        return Functions.convert_to_type(self.__json_data,"generation-vi",TypeSpritesGenerationvi)

    @property
    def generation_vii(self) -> Union[TypeSpritesGenerationvii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-vii",TypeSpritesGenerationvii)

    @property
    def generation_viii(self) -> Union[TypeSpritesGenerationviii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-viii",TypeSpritesGenerationviii)

    @property
    def generation_ix(self) -> Union[TypeSpritesGenerationix,None]:
        return Functions.convert_to_type(self.__json_data,"generation-ix",TypeSpritesGenerationix)
 
    
class Types(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/type/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def damage_relations(self) -> Union[TypeRelations,None]:
        return Functions.convert_to_type(self._json_data,"damage_relations",TypeRelations)
    
    @property
    def past_damage_relations(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"past_damage_relations",TypeRelationsPast)

    @property
    def game_indices(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"game_indices",GenerationGameIndex)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"generation",NamedAPIResource)
    
    @property
    def move_damage_class(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"move_damage_class",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)

    @property
    def pokemon(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon",TypePokemon)
    
    @property
    def moves(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"moves",NamedAPIResource)
    
    @property
    def sprites(self) -> Union[TypeSprites,None]:
        return Functions.convert_to_type(self._json_data,"sprites",TypeSprites)


from ..Utility.Common import *
from typing import List,Union

class PokemonAbility:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def is_hidden(self) -> Union[bool,None]:
        return Functions.convert_to_type(self.__json_data,"is_hidden",bool)
    
    @property
    def slot(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"slot",int)
    
    @property
    def ability(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"ability",NamedAPIResource)
    
class PokemonType:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def slot(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"slot",int)
    
    @property
    def type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"type",NamedAPIResource)
    
class PokemonFormType:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def slot(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"slot",int)
    
    @property
    def type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"type",NamedAPIResource)
    
class PokemonTypePast:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"generation",NamedAPIResource)
    
    @property
    def types(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"types",PokemonType)

class PokemonHeldItem:
    def __init__(self,json_data):
        self.__json_data:dict = json_data
        
    @property
    def item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"item",NamedAPIResource)
    
    @property
    def version_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"version_details",PokemonHeldItemVersion)
    
class PokemonHeldItemVersion:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)
    
    @property
    def rarity(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"rarity",int)
    
class PokemonMove:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def move(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"move",NamedAPIResource)
    
    @property
    def version_group_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"version_group_details",PokemonMoveVersion)
    
class PokemonMoveVersion:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def move_learn_method(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"move_learn_method",NamedAPIResource)

    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
        
    @property
    def level_learned_at(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"level_learned_at",int)
    
class PokemonStat:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"stat",NamedAPIResource)
        
    @property
    def effort(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"effort",int)
    
    @property
    def base_stat(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"base_stat",int)

class PokemonSpritesRedBlueYellow:
    def __init__(self,json_data):
        self.__json_data = json_data
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_gray(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_gray",str)
    
    @property
    def back_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_transparent",str)
    
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_gray(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_gray",str)
    
    @property
    def front_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_transparent",str)
    
class PokemonSpritesCrystal:
    def __init__(self,json_data):
        self.__json_data = json_data
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_shiny_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_transparent",str)
    
    @property
    def back_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_transparent",str)
    
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_shiny_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_transparent",str)
    
    @property
    def front_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_transparent",str)

class PokemonSpritesGoldSilver:
    def __init__(self,json_data):
        self.__json_data = json_data
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_transparent(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_transparent",str)
    
class PokemonSpritesEmerald:
    def __init__(self,json_data):
        self.__json_data = json_data
    
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
class PokemonSpritesFRRG:
    def __init__(self,json_data):
        self.__json_data = json_data

    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
     
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
class PokemonSpritesRS:
    def __init__(self,json_data):
        self.__json_data = json_data

    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
     
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
class PokemonSpritesDP:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)
    
class PokemonSpritesHGSS:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)

class PokemonSpritesPt:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)
    
class PokemonSpritesAnimated:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)
    
class PokemonSpritesBW:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def animated(self) -> Union[PokemonSpritesAnimated,None]:
        return Functions.convert_to_type(self.__json_data,"animated",PokemonSpritesAnimated)
     
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)

class PokemonSpritesORAS:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
class PokemonSpritesXY:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)

class PokemonSpritesUSUM:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)

class PokemonSpritesIcons:
    def __init__(self,json_data):
        self.__json_data = json_data
       
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)

    
class PokemonSpritesVersionGi:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def red_blue(self) -> Union[PokemonSpritesRedBlueYellow,None]:
        return Functions.convert_to_type(self.__json_data,"red-blue",PokemonSpritesRedBlueYellow)
    
    @property
    def yellow(self) -> Union[PokemonSpritesRedBlueYellow,None]:
        return Functions.convert_to_type(self.__json_data,"yellow",PokemonSpritesRedBlueYellow)

class PokemonSpritesVersionGii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def crystal(self) -> Union[PokemonSpritesCrystal,None]:
        return Functions.convert_to_type(self.__json_data,"crystal",PokemonSpritesCrystal)
    
    @property
    def gold(self) -> Union[PokemonSpritesGoldSilver,None]:
        return Functions.convert_to_type(self.__json_data,"gold",PokemonSpritesGoldSilver)

    @property
    def silver(self) -> Union[PokemonSpritesGoldSilver,None]:
        return Functions.convert_to_type(self.__json_data,"silver",PokemonSpritesGoldSilver)

class PokemonSpritesVersionGiii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def emerald(self) -> Union[PokemonSpritesEmerald,None]:
        return Functions.convert_to_type(self.__json_data,"emerald",PokemonSpritesEmerald)
    
    @property
    def firered_leafgreen(self) -> Union[PokemonSpritesFRRG,None]:
        return Functions.convert_to_type(self.__json_data,"firered-leafgreen",PokemonSpritesFRRG)

    @property
    def ruby_sapphire(self) -> Union[PokemonSpritesRS,None]:
        return Functions.convert_to_type(self.__json_data,"ruby-sapphire",PokemonSpritesRS)

class PokemonSpritesVersionGiv:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def diamond_pearl(self) -> Union[PokemonSpritesDP,None]:
        return Functions.convert_to_type(self.__json_data,"diamond-pearl",PokemonSpritesDP)
    
    @property
    def heartgold_soulsilver(self) -> Union[PokemonSpritesHGSS,None]:
        return Functions.convert_to_type(self.__json_data,"heartgold-soulsilver",PokemonSpritesHGSS)

    @property
    def platinum(self) -> Union[PokemonSpritesPt,None]:
        return Functions.convert_to_type(self.__json_data,"platinum",PokemonSpritesPt)
       
class PokemonSpritesVersionGv:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def black_white(self) -> Union[PokemonSpritesBW,None]:
        return Functions.convert_to_type(self.__json_data,"black-white",PokemonSpritesBW)
    
class PokemonSpritesVersionGvi:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def omegaruby_alphasapphire(self) -> Union[PokemonSpritesORAS,None]:
        return Functions.convert_to_type(self.__json_data,"omegaruby-alphasapphire",PokemonSpritesORAS)
    
    @property
    def x_y(self) -> Union[PokemonSpritesXY,None]:
        return Functions.convert_to_type(self.__json_data,"x-y",PokemonSpritesXY)
    
class PokemonSpritesVersionGvii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def icons(self) -> Union[PokemonSpritesIcons,None]:
        return Functions.convert_to_type(self.__json_data,"icons",PokemonSpritesIcons)
    
    @property
    def ultra_sun_ultra_moon(self) -> Union[PokemonSpritesUSUM,None]:
        return Functions.convert_to_type(self.__json_data,"ultra-sun-ultra-moon",PokemonSpritesUSUM)
    
class PokemonSpritesVersionGviii:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def icons(self) -> Union[PokemonSpritesIcons,None]:
        return Functions.convert_to_type(self.__json_data,"icons",PokemonSpritesIcons)
    
class PokemonSpritesVersion:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def generation_i(self) -> Union[PokemonSpritesVersionGi,None]:
        return Functions.convert_to_type(self.__json_data,"generation-i",PokemonSpritesVersionGi)

    @property
    def generation_ii(self) -> Union[PokemonSpritesVersionGii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-ii",PokemonSpritesVersionGii)

    @property
    def generation_iii(self) -> Union[PokemonSpritesVersionGiii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-iii",PokemonSpritesVersionGiii)

    @property
    def generation_iv(self) -> Union[PokemonSpritesVersionGiv,None]:
        return Functions.convert_to_type(self.__json_data,"generation-iv",PokemonSpritesVersionGiv)

    @property
    def generation_v(self) -> Union[PokemonSpritesVersionGv,None]:
        return Functions.convert_to_type(self.__json_data,"generation-v",PokemonSpritesVersionGv)

    @property
    def generation_vi(self) -> Union[PokemonSpritesVersionGvi,None]:
        return Functions.convert_to_type(self.__json_data,"generation-vi",PokemonSpritesVersionGvi)

    @property
    def generation_vii(self) -> Union[PokemonSpritesVersionGvii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-vii",PokemonSpritesVersionGvii)

    @property
    def generation_viii(self) -> Union[PokemonSpritesVersionGviii,None]:
        return Functions.convert_to_type(self.__json_data,"generation-viii",PokemonSpritesVersionGviii)

class PokemonSprites2:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)

class PokemonSpritesOther:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def dream_world(self) -> Union[PokemonSprites2,None]:
        return Functions.convert_to_type(self.__json_data,"dream_world",PokemonSprites2)
    
    @property
    def home(self) -> Union[PokemonSprites2,None]:
        return Functions.convert_to_type(self.__json_data,"home",PokemonSprites2)
    
    @property
    def official_artwork(self) -> Union[PokemonSprites2,None]:
        return Functions.convert_to_type(self.__json_data,"official-artwork",PokemonSprites2)
    
    @property
    def showdown(self) -> Union[PokemonSprites2,None]:
        return Functions.convert_to_type(self.__json_data,"showdown",PokemonSprites2)
    
class PokemonSprites:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)
    
    @property
    def other(self) -> Union[PokemonSpritesOther,None]:
        return Functions.convert_to_type(self.__json_data,"other",PokemonSpritesOther)
    
    @property
    def versions(self) -> Union[PokemonSpritesVersion,None]:
        return Functions.convert_to_type(self.__json_data,"versions",PokemonSpritesVersion)

class PokemonCries:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def latest(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"latest",str)
        
    @property
    def legacy(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"legacy",str)
    
class Pokemon(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokemon/" + str(id))
        
    @property
    def id(self)-> Union[int , None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str , None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def base_experience(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"base_experience",int)
    
    @property
    def height(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"height",int)

    @property
    def is_default(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_default",bool)

    @property
    def order(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"order",int)

    @property
    def weight(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"weight",int)

    @property
    def abilities(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"abilities",PokemonAbility)
    
    @property
    def forms(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"forms",NamedAPIResource)
    
    @property
    def game_indices(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"game_indices",VersionGameIndex)
    
    @property
    def held_items(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"held_items",PokemonHeldItem)
    
    @property
    def location_area_encounters(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"location_area_encounters",str)
    @property
    def moves(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"moves",PokemonMove)
    
    @property
    def past_types(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"past_types",PokemonTypePast)
    
    @property
    def sprites(self) -> Union[PokemonSprites,None]:
        return Functions.convert_to_type(self._json_data,"sprites",PokemonSprites)
    
    @property
    def cries(self) -> Union[PokemonCries,None]:
        return Functions.convert_to_type(self._json_data,"cries",PokemonCries)
    
    @property
    def species(self) -> Union[NamedAPIResource , None]:
        return Functions.convert_to_type(self._json_data,"species",NamedAPIResource)

    @property
    def stats(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"stats",PokemonStat)
    
    @property
    def types(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"types",PokemonType)


from ..Utility.Common import *
from typing import List

class ItemPocket(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/item-fling-effect/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def categories(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"categories",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    


from ..Utility.Common import *
from typing import List

class ContestEffect(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/contest-effect/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def appeal(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"appeal",int)
    
    @property
    def jam(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"jam",int)
    
    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"effect_entries",Effect)
    
    @property
    def flavor_text_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavor_text_entries",FlavorText)



from ..Utility.Common import *
from typing import List

class PokemonSpecies(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokemon-species/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def order(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"order",int)
    
    @property
    def gender_rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"gender_rate",int)
    
    @property
    def capture_rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"capture_rate",int)
    
    @property
    def base_happiness(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"base_happiness",int)
    
    @property
    def is_baby(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_baby",bool)
    
    @property
    def is_legendary(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_legendary",bool)
    
    @property
    def is_mythical(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_mythical",bool)
    
    @property
    def hatch_counter(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"hatch_counter",int)

    @property
    def has_gender_differences(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"has_gender_differences",bool)
    
    @property
    def forms_switchable(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"forms_switchable",bool)
    
    @property
    def growth_rate(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"growth_rate",NamedAPIResource)
    
    @property
    def pokedex_numbers(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokedex_numbers",PokemonSpeciesDexEntry)
    
    @property
    def egg_groups(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"egg_groups",NamedAPIResource)
    
    @property
    def color(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"color",NamedAPIResource)
    
    @property
    def shape(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"shape",NamedAPIResource)
    
    @property
    def evolves_from_species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"evolves_from_species",NamedAPIResource)
    
    @property
    def evolution_chain(self) -> Union[APIResource,None]:
        return Functions.convert_to_type(self._json_data,"evolution_chain",APIResource)
    
    @property
    def habitat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"habitat",NamedAPIResource)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"generation",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def pal_park_encounters(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pal_park_encounters",PalParkEncounterArea)
    
    @property
    def flavor_text_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavor_text_entries",FlavorText)
    
    @property
    def form_descriptions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"form_descriptions",Description)
    
    @property
    def genera(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"genera",Genus)
    
    @property
    def varieties(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"varieties",PokemonSpeciesVariety)
    
    
class Genus:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def genus(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"genus",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
class PokemonSpeciesDexEntry:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def entry_number(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"entry_number",int)
    
    @property
    def pokedex(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokedex",NamedAPIResource)
    
class PalParkEncounterArea:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def base_score(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"base_score",int)
    @property
    def rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"rate",int)
    @property
    def area(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"area",NamedAPIResource)
    
class PokemonSpeciesVariety:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def is_default(self) -> Union[bool,None]:
        return Functions.convert_to_type(self.__json_data,"is_default",bool)

    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon",NamedAPIResource)


from .Location import Location
from .LocationArea import LocationArea
from .PalParkArea import PalParkArea
from .Region import Region

__all__ = ["Location",
           "LocationArea",
           "PalParkArea",
           "Region"]


from ..Utility.Common import *
from typing import List
class Genders(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/gender/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def pokemon_species_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon_species_details",PokemonSpeciesGender)
    
    @property
    def required_for_evolution(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"required_for_evolution",NamedAPIResource)
    
    
class PokemonSpeciesGender:
    def __init__(self,json_data):
        self.__json_data = json_data
        

    @property
    def rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"rate",int)
    
    @property
    def pokemon_species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon_species",NamedAPIResource)


from ..Utility.Common import *
from typing import List

class NaturePokeathlonStatAffectSets:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def increase(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"increase",NaturePokeathlonStatAffect)
    
    @property
    def decrease(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"decrease",NaturePokeathlonStatAffect)
      
class NaturePokeathlonStatAffect:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def max_change(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_change",int)
    
    @property
    def nature(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"nature",NamedAPIResource)
    
class PokeathlonStats(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokeathlon-stat/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def affecting_natures(self) -> Union[NaturePokeathlonStatAffectSets,None]:
        return Functions.convert_to_type(self._json_data,"affecting_natures",NaturePokeathlonStatAffectSets)

 
    


from .Generation import Generation
from .Pokedex import Pokedex
from .Version import Version
from .VersionGroup import VersionGroup

__all__ = ["Generation",
           "Pokedex",
           "Version",
           "VersionGroup"]


from ..Utility.Common import *
from typing import List
class Natures(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/nature/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def decreased_stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"decreased_stat",NamedAPIResource)
    
    @property
    def increased_stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"increased_stat",NamedAPIResource)

    @property
    def hates_flavor(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"hates_flavor",NamedAPIResource)

    @property
    def likes_flavor(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"likes_flavor",NamedAPIResource)
    
    @property
    def pokeathlon_stat_changes(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokeathlon_stat_changes",NatureStatChange)
    
    @property
    def move_battle_style_preferences(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"move_battle_style_preferences",MoveBattleStylePreference)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
class NatureStatChange:
    def __init__(self,json_data):
        self.__json_data = json_data
        

    @property
    def max_change(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_change",int)
    
    @property
    def pokeathlon_stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokeathlon_stat",NamedAPIResource)
    
class MoveBattleStylePreference:
    def __init__(self,json_data):
        self.__json_data = json_data
        

    @property
    def low_hp_preference(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"low_hp_preference",int)
    
    @property
    def high_hp_preference(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"high_hp_preference",int)
    
    @property
    def move_battle_style(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"move_battle_style",NamedAPIResource)


from ..Utility.Common import *
from typing import List

class Machine(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/machine/" + str(id))
        
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"item",NamedAPIResource)
  
    @property
    def move(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"move",NamedAPIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"version_group",NamedAPIResource)


from ..Utility.Common import *
from typing import List

class LocationArea(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/location-area/" + str(id))
        
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def game_index(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"game_index",int)
    
    @property
    def encounter_method_rates(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"encounter_method_rates",EncounterMethodRate)

    @property
    def location(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"location",NamedAPIResource)
    
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def pokemon_encounters(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon_encounters",PokemonEncounter)

class EncounterMethodRate:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def encounter_method(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"encounter_method",NamedAPIResource)
    
    @property
    def version_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"version_details",EncounterVersionDetails)
    
class EncounterVersionDetails:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"rate",int)
    
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)
    
class PokemonEncounter:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon",NamedAPIResource)
    
    @property
    def version_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"version_details",VersionEncounterDetail)
    



from ..Utility.Common import *
from typing import List

class ItemSprites:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"default",str)
    
class ItemHolderPokemon:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon",NamedAPIResource)
    
    @property
    def version_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"version_details",ItemHolderPokemonVersionDetail)
    
class ItemHolderPokemonVersionDetail:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def rarity(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"rarity",int)
    
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)

class Item(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/item/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def cost(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"cost",int)
    
    @property
    def fling_power(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"fling_power",int)
    
    @property
    def fling_effect(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"fling_effect",NamedAPIResource)
    
    @property
    def attributes(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"attributes",NamedAPIResource)
    
    @property
    def category(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"category",NamedAPIResource)
    
    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"effect_entries",VerboseEffect)
    
    @property
    def flavor_text_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavor_text_entries",VersionGroupFlavorText)

    @property
    def game_indices(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"game_indices",GenerationGameIndex)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def sprites(self) -> Union[ItemSprites,None]:
        return Functions.convert_to_type(self._json_data,"sprites",ItemSprites)
    
    @property
    def held_by_pokemon(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"held_by_pokemon",ItemHolderPokemon)
    
    @property
    def baby_trigger_for(self) -> Union[APIResource,None]:
        return Functions.convert_to_type(self._json_data,"baby_trigger_for",APIResource)
    
    @property
    def machines(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"machines",MachineVersionDetail)
     

    


from ..Utility.Common import *
from typing import List
class GrowthRate(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/growth-rate/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def formula(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"formula",str)
    
    @property
    def descriptions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"descriptions",Description)
    
    @property
    def levels(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"levels",GrowthRateExperienceLevel)
    
    @property
    def pokemon_species(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon_species",NamedAPIResource)
    
    
class GrowthRateExperienceLevel:
    def __init__(self,json_data):
        self.__json_data = json_data
        

    @property
    def level(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"level",int)
    
    @property
    def experience(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"experience",int)
    


from .Pokemon import Pokemon
from .Abilities import Abilities
from .Characteristic import Characteristic
from .EggGroup import EggGroup
from .Genders import Genders
from .GrowthRate import GrowthRate
from .Natures import Natures
from .PokeathlonStats import PokeathlonStats
from .PokemonColors import PokemonColors
from .PokemonForms import PokemonForms
from .PokemonHabitats import PokemonHabitats
from .PokemonLocationAreas import PokemonLocationAreas
from .PokemonShapes import PokemonShapes
from .PokemonSpecies import PokemonSpecies
from .Stats import Stats
from .Types import Types

__all__ = ['Pokemon',
           'Abilities',
           'Characteristic',
           'EggGroup',
           'Genders',
           'GrowthRate',
           'Natures',
           'PokeathlonStats',
           'PokemonColors',
           'PokemonForms',
           'PokemonHabitats',
           'PokemonLocationAreas',
           'PokemonShapes',
           'PokemonSpecies',
           'Stats',
           'Types']


from ..Utility.Common import *
from typing import List
class PokemonLocationAreas(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokemon/" + str(id) + "/encounters")
        
    @property
    def location_area(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"location_area",NamedAPIResource)
    
    
    @property
    def version_details(self) -> Union[List,None]:
        return Functions.convert_to_type(self._json_data,"version_details",VersionEncounterDetail)
    


from .EncounterMethod import EncounterMethod
from .EncounterCondition import EncounterCondition
from .EncounterConditionValue import EncounterConditionValue


__all__ = ["EncounterMethod",
           "EncounterCondition",
           "EncounterConditionValue"]


from ..Utility.Common import *
from typing import List
class MoveStatAffectSets:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def increase(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"increase",MoveStatAffect)
    
    @property
    def decrease(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"decrease",MoveStatAffect)
      
class MoveStatAffect:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def change(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"change",int)
    
    @property
    def move(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"move",NamedAPIResource)
    
    
class NatureStatAffectSets:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def increase(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"increase",NamedAPIResource)
    
    @property
    def decrease(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"decrease",NamedAPIResource)

class Stats(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/stat/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def game_index(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"game_index",int)
    
    @property
    def is_battle_only(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_battle_only",bool)
    
    @property
    def affecting_moves(self) -> Union[MoveStatAffectSets,None]:
        return Functions.convert_to_type(self._json_data,"affecting_moves",MoveStatAffectSets)
    
    @property
    def affecting_natures(self) -> Union[NatureStatAffectSets,None]:
        return Functions.convert_to_type(self._json_data,"affecting_natures",NatureStatAffectSets)
    
    @property
    def characteristics(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"characteristics",APIResource)
    
    @property
    def move_damage_class(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"move_damage_class",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)




from ..Utility.Common import *
from typing import List

class ContestType(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/contest-type/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def berry_flavor(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"berry_flavor",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",ContestName)

class ContestName:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"name",str)
    
    @property
    def color(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"color",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    


from .Moves import Moves
from .MoveAilments import MoveAilments
from .MoveBattleStyles import MoveBattleStyles
from .MoveCategory import MoveCategory
from .MoveDamageClass import MoveDamageClass
from .MoveLearnMethod import MoveLearnMethod
from .MoveTarget import MoveTarget

__all__ = ["Moves",
           "MoveAilments",
           "MoveBattleStyles",
           "MoveCategory",
           "MoveDamageClass",
           "MoveLearnMethod",
           "MoveTarget"]


from ..Utility.Common import *
from typing import List
class PokemonFormSprites:
    def __init__(self,json_data):
        self.__json_data = json_data
        

    @property
    def front_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_default",str)

    @property
    def front_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_female",str)
    
    @property
    def front_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny",str)
    
    @property
    def front_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"front_shiny_female",str)
    
    @property
    def back_default(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_default",str)
    
    @property
    def back_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_female",str)
    
    @property
    def back_shiny(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny",str)
    
    @property
    def back_shiny_female(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"back_shiny_female",str)
    
class PokemonForms(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokemon-form/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def order(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"order",int)
    
    @property
    def form_order(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"form_order",int)
    
    @property
    def is_default(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_default",bool)
    
    @property
    def is_battle_only(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_battle_only",bool)
    
    @property
    def is_mega(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_mega",bool)

    @property
    def form_name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"form_name",str)
    
    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"pokemon",NamedAPIResource)
    
    @property
    def types(self) -> Union[List,None]:
        from .Pokemon import PokemonFormType
        return Functions.convert_to_type_list(self._json_data,"types",PokemonFormType)
    
    @property
    def sprites(self) -> Union[PokemonFormSprites,None]:
        return Functions.convert_to_type(self._json_data,"sprites",PokemonFormSprites)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"version_group",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def form_names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"form_names",Name)
    
    

    


from ..Utility.Common import *
from .BerryFirmness import BerryFirmness
from .BerryFlavor import BerryFlavor
from ..Items.Item import Item
from ..Pokemon.Types import Types
from typing import List

class Berry(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/berry/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def growth_time(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"growth_time",int)
    
    @property
    def max_harvest(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"max_harvest",int)
    
    @property
    def natural_gift_power(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"natural_gift_power",int)
    
    @property
    def size(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"size",int)
    
    @property
    def smoothness(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"smoothness",int)
    
    @property
    def soil_dryness(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"soil_dryness",int)
    
    @property
    def firmness(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"firmness",NamedAPIResource)
    
    @property
    def flavors(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavors",BerryFlavorMap)
    
    @property
    def item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"item",NamedAPIResource)
    
    @property
    def natural_gift_type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"natural_gift_type",NamedAPIResource)
    
class BerryFlavorMap:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def potency(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"potency",int)
    
    @property
    def flavor(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"flavor",NamedAPIResource)
    


from ..Utility.Common import *
from typing import List

class Pokedex(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/pokedex/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def is_main_series(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_main_series",bool)
    
    @property
    def descriptions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"descriptions",Description)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def pokemon_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon_entries",PokemonEntry)
    
    @property
    def region(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"region",NamedAPIResource)
    
    @property
    def version_groups(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"version_groups",NamedAPIResource)
    
    
    
class PokemonEntry:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def entry_number(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"entry_number",int)
    
    @property
    def pokemon_species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon_species",NamedAPIResource)
    


from ..Utility.Common import *
from typing import List
class Characteristic(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/characteristic/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def gene_modulo(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"gene_modulo",int)

    @property
    def possible_values(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"possible_values",int)  
    
    @property
    def highest_stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"highest_stat",NamedAPIResource)
    
    @property
    def descriptions(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"descriptions",Description)  


from .ContestType import ContestType
from .ContestEffect import ContestEffect
from .SuperContestEffect import SuperContestEffect

__all__ = ["ContestType",
           "ContestEffect",
           "SuperContestEffect"]


__author__ = 'atama12'
__email__ = 'atama5860@gmail.com'
__credits__ = ["atama12"]
__version__ = '0.1.16'
__copyright__ = 'Copyright atama12 2024'
__license__ = 'MIT'

from .Berries import Berry,BerryFirmness,BerryFlavor
from .Contests import ContestEffect,ContestType,SuperContestEffect
from .Encounters import EncounterCondition,EncounterConditionValue,EncounterMethod
from .Evolution import EvolutionChain,EvolutionTrigger
from .Games import Version,VersionGroup,Generation,Pokedex
from .Items import Item,ItemAttribute,ItemCategory,ItemFlingEffect,ItemPocket
from .Locations import Location,LocationArea,PalParkArea,Region
from .Machines  import Machine
from .Moves import Moves,MoveAilments,MoveBattleStyles,MoveCategory,MoveDamageClass,MoveLearnMethod,MoveTarget
from .Pokemon import Abilities,Characteristic,EggGroup,Genders,GrowthRate,Natures,PokeathlonStats,PokemonColors,PokemonForms,PokemonHabitats,PokemonLocationAreas,PokemonShapes,Pokemon,PokemonSpecies,Stats,Types
from .Utility import Common,Language


__all__ = ["Berry",
           "BerryFirmness",
           "BerryFlavor",
           "ContestEffect",
           "ContestType",
           "SuperContestEffect",
           "EncounterCondition",
           "EncounterConditionValue",
           "EncounterMethod",
           "EvolutionChain",
           "EvolutionTrigger",
           "Version",
           "VersionGroup",
           "Generation",
           "Pokedex",
           "Item",
           "ItemAttribute",
           "ItemCategory",
           "ItemFlingEffect",
           "ItemPocket",
           "Location",
           "LocationArea",
           "PalParkArea",
           "Region",
           "Machine",
           "Moves",
           "MoveAilments",
           "MoveBattleStyles",
           "MoveCategory",
           "MoveDamageClass",
           "MoveLearnMethod",
           "MoveTarget",
           "Pokemon",
           "Abilities",
           "Characteristic",
           "EggGroup",
           "Genders",
           "GrowthRate",
           "Natures",
           "PokeathlonStats",
           "PokemonColors",
           "PokemonForms",
           "PokemonHabitats",
           "PokemonLocationAreas",
           "PokemonShapes",
           "PokemonSpecies",
           "Stats",
           "Types",
           "Common",
           "Language"]


from .Common import BaseModel,Functions,APIResource,NamedAPIResource,Description,Effect,Encounter,FlavorText,GenerationGameIndex,MachineVersionDetail,Name,VerboseEffect,VersionGameIndex,VersionEncounterDetail,VersionGroupFlavorText
from .Language import Language

__all__ = ["BaseModel",
           "Functions",
           "APIResource",
           "NamedAPIResource",
           "Description",
           "Effect",
           "Encounter",
           "FlavorText",
           "GenerationGameIndex",
           "MachineVersionDetail",
           "Name",
           "VerboseEffect",
           "VersionGameIndex",
           "VersionEncounterDetail",
           "VersionGroupFlavorText",
           "Language"]


from ..Utility.Common import *
from typing import List

class ChainLink:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def is_baby(self) -> Union[bool,None]:
        return Functions.convert_to_type(self.__json_data,"is_baby",bool)
    
    @property
    def species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"species",NamedAPIResource)
    
    @property
    def evolution_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"evolution_details",EvolutionDetail)
    
    @property
    def evolves_to(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"evolves_to",ChainLink)
    
class EvolutionDetail:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"item",NamedAPIResource)
    
    @property
    def trigger(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"trigger",NamedAPIResource)
    
    @property
    def gender(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"gender",int)
    
    @property
    def held_item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"held_item",NamedAPIResource)
    
    @property
    def known_move(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"known_move",NamedAPIResource)
    
    @property
    def known_move_type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"known_move_type",NamedAPIResource)
    
    @property
    def location(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"location",NamedAPIResource)
    
    @property
    def min_level(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_level",int)
  
    @property
    def min_happiness(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_happiness",int)
 
    @property
    def min_beauty(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_beauty",int)
 
    @property
    def min_affection(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_affection",int)
   
    @property
    def needs_overworld_rain(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"needs_overworld_rain",int)
    
    @property
    def party_species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"party_species",NamedAPIResource)
    
    @property
    def party_type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"party_type",NamedAPIResource)
    
    @property
    def relative_physical_stats(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"relative_physical_stats",int)
   
    @property
    def time_of_day(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"time_of_day",str)
   
    @property
    def trade_species(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"trade_species",NamedAPIResource)
   
    @property
    def turn_upside_down(self) -> Union[bool,None]:
        return Functions.convert_to_type(self.__json_data,"turn_upside_down",bool)
   
class EvolutionChain(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/evolution-chain/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def baby_trigger_item(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"baby_trigger_item",NamedAPIResource)
    
    @property
    def chain(self) -> Union[ChainLink,None]:
        return Functions.convert_to_type(self._json_data,"chain",ChainLink)


import requests_cache
from typing import List,Type,Union
# requests_cache を有効化
import requests

class Functions:
    def convert_to_type(json_data:dict,key: str,target_type: Type):
        if key in json_data and json_data.get(key) is not None:
            return target_type(json_data.get(key))
        else:
            return None
        
    def convert_to_type_list(json_data:dict,key: str,target_type: Type):
        if key in json_data and json_data.get(key) is not None:
            array : List[target_type] = [target_type(js) for js in json_data.get(key)]
            return array
        else:
            return None
class BaseModel:
    def __init__(self,url):
        self.session = requests_cache.CachedSession('pokemon',expire_after=86400)
        self._json_data : dict | None = self.make_request(url)
        
    def make_request(self,url):
        response = self.session.get(url)
        self._status_code = response.status_code
        if response.status_code == 200:
            return response.json()
        elif response.status_code == 404:
            print(f"Error: Resource not fount at {url}")
            return None
        else:
            response.raise_for_status()
            
    @property
    def status_code(self):
        return self._status_code
            
            
class APIResource:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def url(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"url",str)
    
class NamedAPIResource:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"name",str)
    
    @property
    def url(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"url",str)
    
class Description:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def description(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"description",str)
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)

class Effect:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def effect(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"effect",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
class Encounter:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def min_level(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_level",int)
    
    @property
    def max_level(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_level",int)
    
    @property
    def condition_values(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"condition_values",NamedAPIResource)
    
    @property
    def chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"chance",int)
    
    @property
    def method(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"method",NamedAPIResource)
    
class FlavorText:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def flavor_text(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"flavor_text",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)
        return NamedAPIResource(self.__json_data["version"])
   
   
class GenerationGameIndex:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def game_index(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"game_index",int)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"generation",NamedAPIResource)

class MachineVersionDetail:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def machine(self) -> Union[APIResource,None]:
        return Functions.convert_to_type(self.__json_data,"machine",APIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
    
 
class Name:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"name",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)


    
class VerboseEffect:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def effect(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"effect",str)
    
    @property
    def short_effect(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"short_effect",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
class VersionGameIndex:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def game_index(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"game_index",int)
    
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)
    
class VersionEncounterDetail:
    def __init__(self,json_data):
        self.__json_data = json_data

    
    @property
    def version(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version",NamedAPIResource)
    
    @property
    def max_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_chance",int)
    
    @property
    def encounter_details(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"encounter_details",Encounter)
    
class VersionGroupFlavorText:
    def __init__(self,json_data):
        self.__json_data = json_data

    
    @property
    def text(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"text",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)



from .Item import Item
from .ItemAttribute import ItemAttribute
from .ItemCategory import ItemCategory
from .ItemFlingEffect import ItemFlingEffect
from .ItemPocket import ItemPocket

__all__ = ["Item",
           "ItemAttribute",
           "ItemCategory",
           "ItemFlingEffect",
           "ItemPocket"]


from setuptools import setup, find_packages
import PyPokeClasses
setup(
    name="PyPokeClasses",  # パッケージ名
    version=PyPokeClasses.__version__,  # バージョン
    author="Atama12",  # 作者名
    author_email="atama5860@gmail.com",  # メールアドレス
    description="A class-based Package for accessing and interacting with PokeAPI's JSON data.",  # パッケージの説明
    long_description=open("README.md", "r",encoding="utf-8").read(),  # 長い説明（READMEから）
    long_description_content_type="text/markdown",  # READMEのフォーマット（markdownの場合）
    url="https://github.com/atama12/PyPokeClasses.git",  # リポジトリのURL
    packages=['PyPokeClasses','PyPokeClasses.Berries','PyPokeClasses.Contests','PyPokeClasses.Encounters','PyPokeClasses.Evolution','PyPokeClasses.Games','PyPokeClasses.Items','PyPokeClasses.Locations','PyPokeClasses.Machines','PyPokeClasses.Moves','PyPokeClasses.Pokemon'],  # パッケージを自動で探してくれる
    package_data={
        "PyPokeClasses": ["py.typed"],  # py.typedを含める
    },
    keywords="PyPokeClasses pypokeclasses pokeAPI",
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3 :: Only',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',  # 対応するPythonのバージョン
    install_requires=[
        "requests",  # 必要なパッケージ
        "typing",
        
    ],
)



from ..Utility.Common import *
from typing import List
class Abilities(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/ability/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def is_main_series(self) -> Union[bool,None]:
        return Functions.convert_to_type(self._json_data,"is_main_series",bool)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"generation",NamedAPIResource)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"effect_entries",VerboseEffect)
    
    @property
    def effect_changes(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"effect_changes",AbilityEffectChange)
    
    @property
    def flavor_text_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavor_text_entries",AbilityFlavorText)
    
    @property
    def pokemon(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"pokemon",AbilityPokemon)
    
    
class AbilityEffectChange:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"effect_entries",Effect)

    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
    
class AbilityFlavorText:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def flavor_text(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"flavor_text",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
    
class AbilityPokemon:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def is_hidden(self) -> Union[bool,None]:
        return Functions.convert_to_type(self.__json_data,"is_hidden",bool)
    
    @property
    def slot(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"slot",int)
    
    @property
    def pokemon(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"pokemon",NamedAPIResource)


from ..Utility.Common import *
from typing import List

class ContestComboDetail:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def use_before(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"use_before",NamedAPIResource)
    
    @property
    def use_after(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"use_after",NamedAPIResource)
    
class ContestComboSets:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def normal(self) -> Union[ContestComboDetail,None]:
        return Functions.convert_to_type(self.__json_data,"normal",ContestComboDetail)
    
    @property
    def super(self) -> Union[ContestComboDetail,None]:
        return Functions.convert_to_type(self.__json_data,"super",ContestComboDetail)

    
class MoveFlavorText:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def flavor_text(self) -> Union[str,None]:
        return Functions.convert_to_type(self.__json_data,"flavor_text",str)
    
    @property
    def language(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"language",NamedAPIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
    
class MoveMetaData:
    def __init__(self,json_data):
        self.__json_data = json_data
    
    @property
    def ailment(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"ailment",NamedAPIResource)
    
    @property
    def category(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"category",NamedAPIResource)
    
    @property
    def min_hits(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_hits",int)
    
    @property
    def max_hits(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_hits",int)
    
    @property
    def min_turns(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"min_turns",int)
    
    @property
    def max_turns(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"max_turns",int)
    
    @property
    def drain(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"drain",int)
    
    @property
    def healing(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"healing",int)
    
    @property
    def crit_rate(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"crit_rate",int)
    
    @property
    def ailment_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"ailment_chance",int)
    
    @property
    def flinch_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"flinch_chance",int)
    
    @property
    def stat_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"stat_chance",int)
    
    
class MoveStatChange:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def change(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"change",int)
    
    @property
    def stat(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"stat",NamedAPIResource)
    
class PastMoveStatValues:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def accuracy(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"accuracy",int)
    
    @property
    def effect_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"effect_chance",int)
    
    @property
    def power(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"power",int)
    
    @property
    def pp(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"pp",int)
    

    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self.__json_data,"effect_entries",VerboseEffect)
    
    @property
    def type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"type",NamedAPIResource)
    
    @property
    def version_group(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"version_group",NamedAPIResource)
    
class Moves(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/move/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)
    
    @property
    def accuracy(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"accuracy",int)
    
    @property
    def effect_chance(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"effect_chance",int)
    
    @property
    def pp(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"pp",int)
    
    @property
    def priority(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"priority",int)
    
    @property
    def power(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"power",int)
    
    @property
    def contest_combos(self) -> Union[ContestComboSets,None]:
        return Functions.convert_to_type(self._json_data,"contest_combos",ContestComboSets)
    
    @property
    def contest_type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"contest_type",NamedAPIResource)

    @property
    def contest_effect(self) -> Union[APIResource,None]:
        return Functions.convert_to_type(self._json_data,"contest_effect",APIResource)
    
    @property
    def damage_class(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"damage_class",NamedAPIResource)
      
    @property
    def effect_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"effect_entries",VerboseEffect)
    
    @property
    def effect_changes(self) -> Union[List,None]:
        from ..Pokemon.Abilities import AbilityEffectChange
        return Functions.convert_to_type_list(self._json_data,"effect_changes",AbilityEffectChange)
    
    @property
    def learned_by_pokemon(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"learned_by_pokemon",NamedAPIResource)
    
    @property
    def flavor_text_entries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"flavor_text_entries",MoveFlavorText)
    
    @property
    def generation(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"generation",NamedAPIResource)

    @property
    def machines(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"machines",MachineVersionDetail)
    
    @property
    def meta(self) -> Union[MoveMetaData,None]:
        return Functions.convert_to_type(self._json_data,"meta",MoveMetaData)
    
    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
    @property
    def past_values(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"past_values",PastMoveStatValues)
    
    @property
    def stat_changes(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"stat_changes",MoveStatChange)
    
    @property
    def super_contest_effect(self) -> Union[APIResource,None]:
        return Functions.convert_to_type(self._json_data,"super_contest_effect",APIResource)
    
    @property
    def target(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"target",NamedAPIResource)
    
    @property
    def type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"type",NamedAPIResource)




from ..Utility.Common import *
from typing import List

class BerryFlavor(BaseModel):
    def __init__(self,id):
        super().__init__("https://pokeapi.co/api/v2/berry-flavor/" + str(id))
        
    @property
    def id(self) -> Union[int,None]:
        return Functions.convert_to_type(self._json_data,"id",int)
    
    @property
    def name(self) -> Union[str,None]:
        return Functions.convert_to_type(self._json_data,"name",str)

    @property
    def berries(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"berries",FlavorBerryMap)
    
    @property
    def contest_type(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self._json_data,"contest_type",NamedAPIResource)

    @property
    def names(self) -> Union[List,None]:
        return Functions.convert_to_type_list(self._json_data,"names",Name)
    
class FlavorBerryMap:
    def __init__(self,json_data):
        self.__json_data = json_data
        
    @property
    def potency(self) -> Union[int,None]:
        return Functions.convert_to_type(self.__json_data,"potency",int)
    
    @property
    def berry(self) -> Union[NamedAPIResource,None]:
        return Functions.convert_to_type(self.__json_data,"berry",NamedAPIResource)
    